data "aws_region" "current" {}

data "aws_caller_identity" "current" {}

data "aws_subnet" "selected" {
  for_each = toset(var.subnet_ids)
  id       = each.value
}

locals {
  ebs_enabled         = var.es_cluster_ebs_volume_size > 0 ? true : false
  es_cluster_ebs_iops = var.es_cluster_ebs_volume_type == "io1" ? var.es_cluster_ebs_iops : 0

  zones             = [for s in data.aws_subnet.selected : s.availability_zone]
  zone_count        = length(distinct(local.zones))
  zone_count_actual = local.zone_count >= 3 ? 3 : local.zone_count

  index_slow_logs_enabled     = var.log_retention_index_slow_logs != null ? true : false
  search_slow_logs_enabled    = var.log_retention_search_slow_logs != null ? true : false
  es_application_logs_enabled = var.log_retention_es_application_logs != null ? true : false
  audit_logs_enabled          = var.log_retention_audit_logs != null ? true : false
}

data "aws_iam_policy_document" "default" {
  statement {
    actions = ["es:*"]

    principals {
      type        = "*"
      identifiers = ["*"]
    }

    resources = [
      "${var.aws_arn_prefix}:es:${data.aws_region.current.name}:${data.aws_caller_identity.current.account_id}:domain/${var.cluster_domain_name}/*"
    ]
  }
}

# ----------------------------------------
# Cloudwatch Log Group to use for
# domain logging
# ----------------------------------------

resource "aws_cloudwatch_log_group" "index_slow_logs" {
  count = local.index_slow_logs_enabled ? 1 : 0

  name_prefix       = format("%s-%s-", var.cluster_domain_name, "index_slow_logs")
  retention_in_days = var.log_retention_index_slow_logs
  tags              = var.tags
}

resource "aws_cloudwatch_log_group" "search_slow_logs" {
  count = local.search_slow_logs_enabled ? 1 : 0

  name_prefix       = format("%s-%s-", var.cluster_domain_name, "search_slow_logs")
  retention_in_days = var.log_retention_search_slow_logs
  tags              = var.tags
}

resource "aws_cloudwatch_log_group" "es_application_logs" {
  count = local.es_application_logs_enabled ? 1 : 0

  name_prefix       = format("%s-%s-", var.cluster_domain_name, "es_application_logs")
  retention_in_days = var.log_retention_es_application_logs
  tags              = var.tags
}

resource "aws_cloudwatch_log_group" "audit_logs" {
  count = local.audit_logs_enabled ? 1 : 0

  name_prefix       = format("%s-%s-", var.cluster_domain_name, "audit_logs")
  retention_in_days = var.log_retention_audit_logs
  tags              = var.tags
}

# ----------------------------------------
# Elasticsearch Domain
# ----------------------------------------

## 2022/09 - This domain depends on a service-linked role existing for elasticsearch. In some
## cases, this already exists, but not all. Only one can exist. So, the account owner needs
## to seed one before this chart can produce resources successfully. Currently, this is
## managed in the gitlab.bit9.local/mcs2/core-infrastructure project.

resource "aws_elasticsearch_domain" "cluster" {

  lifecycle {
    ignore_changes = [
      advanced_security_options,
    ]
  }

  domain_name           = var.cluster_domain_name
  elasticsearch_version = var.es_version

  #Required since we enable basic auth
  access_policies = data.aws_iam_policy_document.default.json

  # Required to set this for TF -- it keeps trying to set it to null. (https://github.com/cloudposse/terraform-aws-elasticsearch/issues/81)
  advanced_options = {
    "rest.action.multi.allow_explicit_index" = true
    "indices.query.bool.max_clause_count"    = var.indices_query_bool_max_clause_count
  }

  advanced_security_options {
    enabled                        = true
    internal_user_database_enabled = true
    master_user_options {
      master_user_name     = var.es_cluster_master_user_name
      master_user_password = var.es_cluster_master_user_password
    }
  }

  cluster_config {
    instance_type  = var.es_cluster_instance_type
    instance_count = length(var.subnet_ids) * var.es_cluster_instance_count

    dedicated_master_enabled = var.es_cluster_dedicated_master_enabled
    dedicated_master_type    = var.es_cluster_dedicated_master_type
    dedicated_master_count   = var.es_cluster_dedicated_master_count

    warm_enabled = var.es_cluster_warm_enabled
    warm_count   = var.es_cluster_warm_count
    warm_type    = var.es_cluster_warm_type

    zone_awareness_enabled = (local.zone_count_actual > 1) ? true : false

    dynamic "zone_awareness_config" {
      for_each = (local.zone_count_actual > 1) ? [local.zone_count_actual] : []

      content {
        availability_zone_count = zone_awareness_config.value
      }
    }
  }

  dynamic "vpc_options" {
    for_each = (var.subnet_ids != []) ? [true] : []

    content {
      subnet_ids         = var.subnet_ids
      security_group_ids = var.security_group_ids
    }
  }

  # https://docs.aws.amazon.com/elasticsearch-service/latest/developerguide/es-configuration-api.html#es-configuration-api-datatypes-domainendpointoptions
  domain_endpoint_options {
    enforce_https       = true
    tls_security_policy = "Policy-Min-TLS-1-2-2019-07"
  }

  ebs_options {
    ebs_enabled = local.ebs_enabled
    volume_size = local.ebs_enabled ? var.es_cluster_ebs_volume_size : null
    volume_type = local.ebs_enabled ? var.es_cluster_ebs_volume_type : null
    iops        = local.ebs_enabled ? local.es_cluster_ebs_iops : null
  }

  encrypt_at_rest {
    enabled = true
  }

  dynamic "log_publishing_options" {
    for_each = local.index_slow_logs_enabled ? [1] : []
    content {
      enabled                  = local.index_slow_logs_enabled
      log_type                 = "INDEX_SLOW_LOGS"
      cloudwatch_log_group_arn = join("", aws_cloudwatch_log_group.index_slow_logs.*.arn)
    }
  }

  dynamic "log_publishing_options" {
    for_each = local.search_slow_logs_enabled ? [1] : []
    content {
      enabled                  = local.search_slow_logs_enabled
      log_type                 = "SEARCH_SLOW_LOGS"
      cloudwatch_log_group_arn = join("", aws_cloudwatch_log_group.search_slow_logs.*.arn)
    }
  }

  dynamic "log_publishing_options" {
    for_each = local.es_application_logs_enabled ? [1] : []
    content {
      enabled                  = local.es_application_logs_enabled
      log_type                 = "ES_APPLICATION_LOGS"
      cloudwatch_log_group_arn = join("", aws_cloudwatch_log_group.es_application_logs.*.arn)
    }
  }

  dynamic "log_publishing_options" {
    for_each = local.audit_logs_enabled ? [1] : []
    content {
      enabled                  = local.audit_logs_enabled
      log_type                 = "AUDIT_LOGS"
      cloudwatch_log_group_arn = join("", aws_cloudwatch_log_group.audit_logs.*.arn)
    }
  }

  node_to_node_encryption {
    enabled = true
  }

  snapshot_options {
    automated_snapshot_start_hour = var.es_cluster_automated_snapshot_start_hour
  }

  timeouts {
    update = "120m"
  }

  tags = var.tags
}
